/********************************************************************************
*                                                                               *
*                R e s o u r c e   W r a p p i n g   U t i l i t y              *
*                                                                               *
*********************************************************************************
* Copyright (C) 1998,2003 by Jeroen van der Zijp.   All Rights Reserved.        *
*********************************************************************************
* This program is free software; you can redistribute it and/or modify          *
* it under the terms of the GNU General Public License as published by          *
* the Free Software Foundation; either version 2 of the License, or             *
* (at your option) any later version.                                           *
*                                                                               *
* This program is distributed in the hope that it will be useful,               *
* but WITHOUT ANY WARRANTY; without even the implied warranty of                *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                 *
* GNU General Public License for more details.                                  *
*                                                                               *
* You should have received a copy of the GNU General Public License             *
* along with this program; if not, write to the Free Software                   *
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    *
*********************************************************************************
* $Id: reswrap.cpp,v 1.6 2003/05/16 19:23:24 fox Exp $                          *
********************************************************************************/
#include "xincs.h"
#include "fxdefs.h"


#define MODE_DECIMAL  0
#define MODE_HEX      1
#define MODE_TEXT     2



/*

  Notes:
  - Updated to version 2.0.
  - Can now also generate output as a text string.
  - When ASCII option is used with text string option, it prints human
    readable C-style string, with non-ASCII characters escaped as appropriate.
  - Removed PPM feature as that was never really used in FOX.
  - License changed to GPL from LGPL because this is a standalone program
    that does not need to be linked to anything.
  - Added MSDOS mode, useful for wrapping ASCII text.
  - Added option to keep extension (separated by '_').
  - Need option to place #include "icons.h" or something into icons.cpp

*/

const char version[]="2.0.0";




/* Print some help */
void printusage(){
  fprintf(stderr,"Usage: reswrap [options] [-o[a] outfile] files...\n");
  fprintf(stderr,"Convert files containing images, text, or binary data into C/C++ data arrays.\n");
  fprintf(stderr,"\n");
  fprintf(stderr,"Options:\n");
  fprintf(stderr,"  -h       Print help\n");
  fprintf(stderr,"  -v       Print version number\n");
  fprintf(stderr,"  -d       Output as decimal\n");
  fprintf(stderr,"  -x       Output as hex (default)\n");
  fprintf(stderr,"  -t[a]    Output as [ascii] text string\n");
  fprintf(stderr,"  -e       Generate external reference declaration\n");
  fprintf(stderr,"  -i       Build an include file\n");
  fprintf(stderr,"  -s       Suppress header in output file\n");
  fprintf(stderr,"  -m       Read files with MS-DOS mode (default is binary)\n");
  fprintf(stderr,"  -k       Keep extension, separated by underscore\n");
  fprintf(stderr,"  -n name  Override resource name\n");
  fprintf(stderr,"  -c cols  Change number of columns in output to cols\n");
  }


/* Print version information */
void printversion(){
  fprintf(stderr,"Reswrap %s %s.\n",version,__DATE__);
  fprintf(stderr,"Copyright (C) 1997,2003 Jeroen van der Zijp. All Rights Reserved.\n");
  fprintf(stderr,"Please visit: http://www.fox-toolkit.org for further information.\n");
  fprintf(stderr,"\n");
  fprintf(stderr,"This program is free software; you can redistribute it and/or modify\n");
  fprintf(stderr,"it under the terms of the GNU General Public License as published by\n");
  fprintf(stderr,"the Free Software Foundation; either version 2 of the License, or\n");
  fprintf(stderr,"(at your option) any later version.\n");
  fprintf(stderr,"\n");
  fprintf(stderr,"This program is distributed in the hope that it will be useful,\n");
  fprintf(stderr,"but WITHOUT ANY WARRANTY; without even the implied warranty of\n");
  fprintf(stderr,"MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n");
  fprintf(stderr,"GNU General Public License for more details.\n");
  fprintf(stderr,"\n");
  fprintf(stderr,"You should have received a copy of the GNU General Public License\n");
  fprintf(stderr,"along with this program; if not, write to the Free Software\n");
  fprintf(stderr,"Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.\n");
  }
  

/* Main */
int main(int argc,char **argv){
  FILE *resfile,*outfile;
  int i,j,k,cc,b,first,col,maxcols,external,header,override,include,mode,colsset,ascii,msdos,keepext,hex;
  char name[100],*filename,*ptr;

  /* Initialize */
  outfile=stdout;
  maxcols=16;
  colsset=0;
  external=0;
  header=1;
  override=0;
  include=0;
  ascii=0;
  mode=MODE_HEX;
  msdos=0;
  keepext=0;

  /* Check arguments */
  if(argc<2){
    printusage();
    exit(1);
    }
    
  /* Process options */
  for(i=1; i<argc; i++){

    /* Option */
    if(argv[i][0]=='-'){

      /* Change output file */
      if(argv[i][1]=='o'){
        i++;
        if(i>=argc){
          fprintf(stderr,"reswrap: missing argument for -o option\n");
          exit(1);
          }
        if(outfile!=stdout) fclose(outfile);
        if(argv[i-1][2]=='a'){
          outfile=fopen(argv[i],"a");
          }
        else{
          outfile=fopen(argv[i],"w");
          }
        if(!outfile){
          fprintf(stderr,"reswrap: unable to open output file %s\n",argv[i]);
          exit(1);
          }
        }

      /* Print help */
      else if(argv[i][1]=='h'){
        printusage();
        exit(0);
        }

      /* Print version */
      else if(argv[i][1]=='v'){
        printversion();
        exit(0);
        }

      /* Switch to decimal */
      else if(argv[i][1]=='d'){
        mode=MODE_DECIMAL;
        if(!colsset) maxcols=16;
        }

      /* Switch to hex */
      else if(argv[i][1]=='x'){
        mode=MODE_HEX;
        if(!colsset) maxcols=16;
        }

      /* Switch to text */
      else if(argv[i][1]=='t'){
        mode=MODE_TEXT;
        if(argv[i][2]=='a') ascii=1;
        if(!colsset) maxcols=80;
        }

      /* Suppress header */
      else if(argv[i][1]=='s'){
        header=0;
        }

      /* Generate as external reference */
      else if(argv[i][1]=='e'){
        external=1;
        }

      /* Building include file implies also extern */
      else if(argv[i][1]=='i'){
        include=1;
        external=1;
        }

      /* Read resource with MS-DOS mode */
      else if(argv[i][1]=='m'){
        msdos=1;
        }

      /* Keep extension */
      else if(argv[i][1]=='k'){
        keepext=1;
        }
        
      /* Change number of columns */
      else if(argv[i][1]=='c'){
        i++;
        if(i>=argc){
          fprintf(stderr,"reswrap: missing argument for -c option\n");
          exit(1);
          }
        if(sscanf(argv[i],"%d",&maxcols)==1 && maxcols<1){
          fprintf(stderr,"reswrap: illegal argument for number of columns\n");
          exit(1);
          }
        colsset=1;
        }

      /* Override resource name */
      else if(argv[i][1]=='n'){
        i++;
        if(i>=argc){
          fprintf(stderr,"reswrap: missing argument for -n option\n");
          exit(1);
          }
        strncpy(name,argv[i],sizeof(name));
        name[sizeof(name)-1]=0;
        override=1;
        }
      }

    /* Resource */
    else{

      /* Get file name */
#ifndef WIN32
      if((filename=strrchr(argv[i],PATHSEP))!=0)
        filename=filename+1;
      else
        filename=argv[i];
#else
      if((filename=strrchr(argv[i],'\\'))!=0)
        filename=filename+1;
      else if((filename=strrchr(argv[i],'/'))!=0) /* For CYGWIN bash */
	filename=filename+1;
      else
        filename=argv[i];
#endif

      /* Output header */
      if(header){
        fprintf(outfile,"/* Generated by reswrap from file %s */\n",filename);
        }

      /* Determine resource name from file name */
      if(!override){
        strncpy(name,filename,sizeof(name));
        name[sizeof(name)-1]=0;
        if(!keepext && (ptr=strrchr(name,'.'))!=0) *ptr=0;
        for(k=j=0; name[j]; j++){
          cc=name[j];
          if(cc=='.') cc='_';
          if(k==0 && !(isalpha(cc) || cc=='_')) continue;
          if(!(isalnum(cc) || cc=='_')) continue;
          name[k++]=cc;
          }
        name[k]=0;
        if(k==0){
          fprintf(stderr,"reswrap: cannot generate resource name from %s\n",filename);
          exit(1);
          }
        }

      /* Open data file; always open as binary */
      resfile=fopen(argv[i],"rb");
      if(!resfile){
        fprintf(stderr,"reswrap: unable to open input file %s\n",argv[i]);
        exit(1);
        }

      /* Generate external reference for #include's */
      if(external){ fprintf(outfile,"extern "); }

      /* Output declaration */
      fprintf(outfile,"const unsigned char %s[]",name);

      /* Generate resource array */
      if(!include){
        if(mode==MODE_TEXT){
          col=0;
          hex=0;
          fprintf(outfile,"=\n  \"");
          while((b=fgetc(resfile))!=EOF){
            if(msdos && (b=='\r')) continue;
            if(col>=maxcols){
              fprintf(outfile,"\"\n  \"");
              col=0;
              }
            if(ascii){
              if(b=='\\'){ fprintf(outfile,"\\\\"); col+=2; hex=0; }
              else if(b=='\a'){ fprintf(outfile,"\\a"); col+=2; hex=0; }
              else if(b=='\t'){ fprintf(outfile,"\\t"); col+=2; hex=0; }
              else if(b=='\r'){ fprintf(outfile,"\\r"); col+=2; hex=0; }
              else if(b=='\f'){ fprintf(outfile,"\\f"); col+=2; hex=0; }
              else if(b=='\v'){ fprintf(outfile,"\\v"); col+=2; hex=0; }
              else if(b=='\"'){ fprintf(outfile,"\\\""); col+=2; hex=0; }
              else if(b=='\n'){ fprintf(outfile,"\\n\"\n  \""); col=0; hex=0; }
              else if(b<32 || b>=127){ fprintf(outfile,"\\x%02x",b); col+=4; hex=1; }
              else if(hex && isxdigit(b)){ fprintf(outfile,"\\x%02x",b); col+=4; hex=1; }
              else{ fprintf(outfile,"%c",b); col+=1; hex=0; }
              }
            else{
              fprintf(outfile,"\\x%02x",b); col+=4;
              }
            }
          fprintf(outfile,"\"\n  ");
          }
        else{
          col=0;
          first=1;
          fprintf(outfile,"={\n  ");
          while((b=fgetc(resfile))!=EOF){
            if(msdos && (b=='\r')) continue;
            if(!first){
              fprintf(outfile,",");
              }
            if(col>=maxcols){
              fprintf(outfile,"\n  ");
              col=0;
              }
            if(mode==MODE_HEX)
              fprintf(outfile,"0x%02x",b);
            else
              fprintf(outfile,"%3d",b);
            first=0;
            col++;
            }
          fprintf(outfile,"\n  }");
          }
        }

      fprintf(outfile,";\n\n");

      fclose(resfile);
      override=0;
      }
    }
  return 0;
  }



